package org.ten60.netkernel.security.endpoint;

import org.netkernel.layer0.meta.impl.SourcedArgumentMetaImpl;
import org.netkernel.layer0.nkf.INKFRequest;
import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.nkf.NKFException;
import org.netkernel.module.standard.endpoint.StandardAccessorImpl;

public class PasswordHashAccessor extends StandardAccessorImpl
{
	private static final int SALT_LEN=8;

	public PasswordHashAccessor()
	{
		declareThreadSafe();
		Class[] sca=new Class[]{String.class};
		declareArgument(new SourcedArgumentMetaImpl("password", null, null, sca));
		declareArgument(new SourcedArgumentMetaImpl("hash", null, null, sca));
		declareArgument(new SourcedArgumentMetaImpl("algorithm", null, "sha512", sca));
		declareSourceRepresentation(Boolean.class);
		declareSourceRepresentation(String.class);
	}

	public void onSource(INKFRequestContext aContext) throws Exception
	{	
		String algorithm=aContext.getThisRequest().argumentExists("algorithm")?aContext.source("arg:algorithm",String.class):"sha512";
		String password=aContext.source("arg:password",String.class);
		String type=aContext.getThisRequest().getArgumentValue("activeType");
		if (type.equals("generatePasswordHash"))
		{	onGeneratePasswordHash(aContext,password,algorithm);
		}
		else if (type.equals("checkPasswordHash"))
		{	onCheckPasswordHash(aContext,password,algorithm);
		}
	}
	
	public void onGeneratePasswordHash(INKFRequestContext aContext, String aPassword, String aAlgorithm) throws Exception
	{	String salt=hash(Double.toString(Math.random()),aContext,aAlgorithm).substring(0,SALT_LEN);
		String hash=hash(aPassword+salt,aContext,aAlgorithm);
		int offset=hash.length()-aPassword.length();
		if (offset<1) offset=1;
		String result= hash.substring(0,offset)+salt+hash.substring(offset);		
		aContext.createResponseFrom(result);
	}

	
	public void onCheckPasswordHash(INKFRequestContext aContext, String aPassword, String aAlgorithm) throws Exception
	{	String hash=aContext.source("arg:hash",String.class);
		int offset=hash.length()-SALT_LEN-aPassword.length();
		if (offset<1) offset=1;
		String salt=hash.substring(offset,offset+SALT_LEN);
		String hash1=hash.substring(0,offset)+hash.substring(offset+SALT_LEN);
		String hash2=hash(aPassword+salt,aContext,aAlgorithm);
		boolean result=hash1.equals(hash2);
		aContext.createResponseFrom(result);
	}
	
	private static String hash(String aValue, INKFRequestContext aContext, String aAlgorithm) throws NKFException
	{	INKFRequest req=aContext.createRequest("active:"+aAlgorithm);
		req.addArgumentByValue("operand",aValue);
		req.setRepresentationClass(String.class);
		return (String)aContext.issueRequest(req);
	}
	
	
}
