package org.ten60.netkernel.security.endpoint;

import java.io.InputStream;
import java.security.KeyStore;
import java.security.PrivateKey;
import java.security.Signature;
import java.security.cert.Certificate;
import java.security.cert.CertificateFactory;

import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.nkf.NKFException;
import org.netkernel.layer0.representation.IReadableBinaryStreamRepresentation;
import org.netkernel.layer0.util.Utils;
import org.netkernel.module.standard.endpoint.StandardAccessorImpl;
import org.ten60.netkernel.security.util.SecurityUtils;

public class SignVerifyStandardAccessor extends StandardAccessorImpl
{	
	public SignVerifyStandardAccessor()
	{	this.declareThreadSafe();
	}

	@Override
	public void onSource(INKFRequestContext context) throws Exception
	{		String type=context.getThisRequest().getArgumentValue("activeType");
			Object result=null;
			IReadableBinaryStreamRepresentation bisa=(IReadableBinaryStreamRepresentation)context.source("arg:operand", IReadableBinaryStreamRepresentation.class);
			String keyPassword=null;
			if(context.getThisRequest().argumentExists("keyPassword"))
			{	keyPassword=(String)context.source("arg:keyPassword", String.class);
			}
			if(type.equals("pkiSignStandard"))
			{	//Sign stream using private key
				KeyStore ks=sourceKeyStore(context);
				String keyID=context.getThisRequest().getArgumentValue("keyID");
				KeyStore.PrivateKeyEntry pkEntry = (KeyStore.PrivateKeyEntry)ks.getEntry(keyID, new KeyStore.PasswordProtection(SecurityUtils.stringToChar(keyPassword)));
				if(pkEntry==null)
				{	throw new NKFException("Key: "+keyID+" not found in keystore");					
				}
				PrivateKey privateKey = pkEntry.getPrivateKey();
				byte[] signature=signStream(privateKey, bisa.getInputStream());
				result=Utils.toHexString(signature);
			}
			else
			{	Certificate publicKeyCert=null;
				if(context.getThisRequest().argumentExists("publicKeyCertificate"))
				{	//Verify using a RFC 1421  Public Key Certificate
					CertificateFactory cf = CertificateFactory.getInstance("X.509");
					IReadableBinaryStreamRepresentation cert=context.source("arg:publicKeyCertificate", IReadableBinaryStreamRepresentation.class);
					publicKeyCert=cf.generateCertificate(cert.getInputStream());
				}
				else
				{	//Verify using public key in a keystore
					KeyStore ks=sourceKeyStore(context);
				 	
					String keyID=context.getThisRequest().getArgumentValue("keyID");
					KeyStore.PasswordProtection ksp=null;
					if(keyPassword!=null)
					{	ksp=new KeyStore.PasswordProtection(SecurityUtils.stringToChar(keyPassword));
					}
					KeyStore.TrustedCertificateEntry pubEntry = (KeyStore.TrustedCertificateEntry)ks.getEntry(keyID, ksp);
					if(pubEntry==null)
					{	throw new NKFException("Key: "+keyID+" not found in keystore");					
					}
					publicKeyCert = pubEntry.getTrustedCertificate();
				}
				String signature=(String)context.source("arg:signature", String.class);
				result=verifyStream(publicKeyCert, bisa.getInputStream(), Utils.fromHexString(signature));
			}
			context.createResponseFrom(result);
	}
	
	private byte[] signStream(PrivateKey privateKey, InputStream is) throws Exception
	{	Signature sig=Signature.getInstance("SHA1withRSA");
		sig.initSign(privateKey);
		updateSignature(sig, is);
		return sig.sign();
	}

	private boolean verifyStream(java.security.cert.Certificate cert, InputStream is, byte[] signature) throws Exception
	{	Signature sig = Signature.getInstance("SHA1withRSA");
		sig.initVerify(cert);
		updateSignature(sig, is);
		return sig.verify(signature);	
	}

	private void updateSignature(Signature s, InputStream is) throws Exception
	{	int a=0;
		while((a=is.available())>0)
		{	byte[] b=new byte[256];
			int read=is.read(b);
			s.update(b,0, read);
		}
	}

	private KeyStore sourceKeyStore(INKFRequestContext context) throws Exception
	{	KeyStore ks = KeyStore.getInstance("JKS");
		IReadableBinaryStreamRepresentation bisa=(IReadableBinaryStreamRepresentation)context.source("arg:keystore", IReadableBinaryStreamRepresentation.class);
		String password=(String)context.source("arg:keystorePassword", String.class);
		ks.load(bisa.getInputStream(), SecurityUtils.stringToChar(password));
		return ks;
	}

}
