package org.ten60.netkernel.security.endpoint;

import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.representation.IHDSNode;
import org.netkernel.layer0.representation.impl.HDSFactory;
import org.netkernel.layer0.util.FastSchematron;
import org.netkernel.layer0.util.PairList;
import org.netkernel.module.standard.endpoint.StandardTransreptorImpl;
import org.ten60.netkernel.security.representation.GateKeeperPolicy;
import org.ten60.netkernel.security.representation.GateKeeperZone;

public class XMLGateKeeperPolicyTransreptor extends StandardTransreptorImpl
{	
	private static final PairList VALIDATION = new PairList(4);
	static
	{	VALIDATION.put("/*[name()!='GateKeeperPolicy']", "MSG_GK_POLICY_ROOT");
		VALIDATION.put("/GateKeeperPolicy/*[name()!='zone']", "MSG_GK_POLICY_ZONE");
		VALIDATION.put("/GateKeeperPolicy/zone/*[name()!='match' and name()!='isValidURI' and name()!='loginURI']", "MSG_GK_POLICY_ZONE_TAGS");
		VALIDATION.put("/GateKeeperPolicy/zone[not(match)]", "MSG_GK_POLICY_ZONE_TAGS");
		VALIDATION.put("/GateKeeperPolicy/zone[not(isValidURI)]", "MSG_GK_POLICY_ZONE_TAGS");
		VALIDATION.put("/GateKeeperPolicy/zone[not(loginURI)]", "MSG_GK_POLICY_ZONE_TAGS");
	}	
	
	public XMLGateKeeperPolicyTransreptor()
	{	super();
		//this.declareFromRepresentation(IReadableBinaryStreamRepresentation.class);
		this.declareToRepresentation(GateKeeperPolicy.class);
		this.declareDescription("converts gatekeeper policy to an internal object representation");
	}
	
	@Override
	public void onTransrept(INKFRequestContext aHelper) throws Exception
	{	
		IHDSNode root=(IHDSNode)aHelper.sourcePrimary(IHDSNode.class);
		
		//Could do XML schema validation here first.
		PairList validation=FastSchematron.validate(HDSFactory.toDOM(root).getDocumentElement(), VALIDATION);
		if (validation.size()>0)
		{	String m = FastSchematron.formatValidationError(validation,aHelper);			
			throw aHelper.createFormattedException("EX_GATEKEEPER_POLICY_PARSE", "MSG_RAW", null,null,m);
		}
		
		GateKeeperPolicy p=new GateKeeperPolicy();
		for(IHDSNode i: root.getNodes("/GateKeeperPolicy/zone"))
		{	GateKeeperZone z=new GateKeeperZone();
			for(IHDSNode j: i.getChildren())
			{	if(j.getName().equals("match"))
				{	z.addMatch((String)j.getValue());
				}
				if(j.getName().equals("isValidURI"))
				{ z.setValidURI((String)j.getValue());
				}
				if(j.getName().equals("loginURI"))
				{	z.setNotValidURI((String)j.getValue());
				}
			}
			p.addZone(z);
		}
		aHelper.createResponseFrom(p);
	}
}